#include "gui_main.h"
//
/*  Save file as gui_main.cpp  */
/*  incomming class name Gui_Main */
//
#include <QCloseEvent>
//
QPointer<Gui_Main> Gui_Main::_self = 0L;
//
Gui_Main* Gui_Main::self( QWidget* parent )
{
	if ( !_self )
	_self = new Gui_Main( parent );
	return _self;
}
//
Gui_Main::Gui_Main( QWidget* parent )
	: QMainWindow( parent )
{
	setupUi( this );
    bind_text->setTabStopWidth(30);
    bind_text->setLineWrapMode(QTextEdit::NoWrap);
	Load_Connector();
    CanResolve(false);
    FormsChecks(false);
}
//
void Gui_Main::Load_Connector()
{  
    ACTUALIDnummer = 0;
    listWidget->clear();
    QDir dira(WORK_CACHEDIR);
    if ( dira.mkpath(WORK_CACHEDIR) ) { } 
    else {
        QMessageBox::critical(0, qApp->tr("Cannot write a cache dir!"),
            qApp->tr("Unable to create a tmp dir on %1").arg(WORK_CACHEDIR), QMessageBox::Cancel);
    }
    
    dbs = new DB_dual(DBFILE_DEFAULT);
    if (!dbs->Connect()) {   /* notice from DB_dual */
    return;
    }
    
    connect(pushButton_5, SIGNAL(clicked()),this, SLOT(AllSerialUpdate()));
    connect(pushButton, SIGNAL(clicked()),this, SLOT(UpdateCurrentDomain()));
    connect(import_bind, SIGNAL(activated()), this , SLOT(ReadBindDirs()));
    connect(pushButton_2, SIGNAL(clicked()), this , SLOT(RemoveCurrent()));
    connect(dbs, SIGNAL(ReloadItemCurrent()), this , SLOT(ReloadCurrent()));
    connect(dbs, SIGNAL(NewItem()), this , SLOT(AppendAliasCurrent()));
    connect(gobrowser, SIGNAL(clicked()), this , SLOT(BrowserCurrent()));

    
     connect(exportbindmasterdir, SIGNAL(activated()), this , SLOT(FillMasterBindDir()));
    
    connect(resolvenames, SIGNAL(activated()), this , SLOT(ResolveAllName()));
    connect(ex_bind_listmaster, SIGNAL(activated()), this , SLOT(FillMasterListFile()));
    connect(ex_bind_listslave, SIGNAL(activated()), this , SLOT(FillSlaveListFile()));
   
    connect(newdoma, SIGNAL(activated()), this , SLOT(NewDomaine()));
    
    
    connect(appabout, SIGNAL(activated()), this , SLOT(AboutApp()));
    
    connect(pushButton_6, SIGNAL(clicked()), this , SLOT(AppendAliasCurrent()));
    
    connect(actionExport_as_Sqlite, SIGNAL(activated()), this , SLOT(SqliteExport()));
    connect(import_sqlite, SIGNAL(activated()), this , SLOT(SqliteImport()));
    
    
    LoadDomainHosts();
    
 #if defined Q_WS_MAC
 #endif
 #if defined Q_WS_WIN
 #endif
 #if defined Q_WS_X11
 #endif
}


void Gui_Main::ResolveAllName()
{
    dbs->FillResolveTableName();
    QTimer::singleShot(1500, this, SLOT(ReloadCurrent()));
    
}


void Gui_Main::RemoveCurrent()
{
    QString host = D_name->text();
    std::cout << "RemoveCurrent " << qPrintable(host)  << " id->" << ACTUALIDnummer << std::endl;
    if (host.size() < 3) {
    return;
    }   
    if (ACTUALIDnummer == 0) {
    return;
    }    
    
    if (dbs->RemoveHost(host,ACTUALIDnummer)) {
    ACTUALIDnummer = 0;
    ResetFormHost();
    LoadDomainHosts();
    }
    
}




void Gui_Main::LoadDomainHosts()
{
    /* rebuild dns host list */
    listWidget->clear();
    FormsChecks(false);
    QStringList namedomains = dbs->HostList();   /* bind */
    for (int i = 0; i <  namedomains.size(); ++i) {
        QListWidgetItem *domx = new QListWidgetItem(QIcon(":/img/info.png"),namedomains.at(i),listWidget);
    }
    listWidget->setSpacing(5);
    connect(listWidget, SIGNAL(clicked(QModelIndex)), this , SLOT(ActiveHost(QModelIndex)));
}


void Gui_Main::PlaceNewItem( DNS_Item * item )
{
    const QString host = item->Get_FullDomain();
    if (!dbs->CheckHost(host)) {
        std::cout << "insert  register " << qPrintable(host) << std::endl;
        QStringList quer = item->Get_Sqlinsert();
        for (int i = 0; i <  quer.size(); ++i) {
            dbs->BaseQuery(quer.at(i));
        }  
    }
    
    item->deleteLater();
    
    /* reload list on 1/2 second */
    LoadDomainHosts();
}


void Gui_Main::ReadBindDirs()
{
    QString dirfromfile = QFileDialog::getExistingDirectory(this, tr("Open Dir from BIND file."),QString(setter.value("LastDir").toString()),QFileDialog::ShowDirsOnly); 
    dirfromfile = dirfromfile.append("/");
    setter.setValue("LastDir",dirfromfile);
    ////////////std::cout << "dirfromfile " << qPrintable(dirfromfile) << std::endl;
    
    if (dirfromfile.size() > 2) {
        
        QStringList hostfiles = dbs->ReadDirBind(dirfromfile);
        for (int i = 0; i <  hostfiles.size(); ++i) {
            const QString filehost = hostfiles.at(i); 
            ParseHostFi *cicle = new ParseHostFi();
            cicle->SetFile(filehost); 
            cicle->start(QThread::LowPriority);
            connect(cicle, SIGNAL(ErrorMsg(QString)), this , SLOT(ErrorImportBind(QString)));
            connect(cicle, SIGNAL(OkJobEnd(DNS_Item *)), this , SLOT(PlaceNewItem(DNS_Item *)));
            /////////////std::cout << "filehost " << qPrintable(filehost) << std::endl;
        }
        
    }
}





void Gui_Main::ActiveHost(const QModelIndex index )
{
    CurrentDomain = "";
    ACTUALIDnummer = 0;
    QString hostgo = index.model()->data(index, Qt::DisplayRole).toString();
    if (hostgo.size() > 0) {
        FormsChecks(true);
        FillNowHost(hostgo);
        CurrentDomain = hostgo;
        
    }
}


void Gui_Main::ReloadCurrent() 
{
    if (CurrentDomain.size() > 3) {
        FillNowHost(CurrentDomain);
    }
}
void Gui_Main::BrowserCurrent() 
{
    if (CurrentDomain.size() > 3) {
        
        OpenDesktop( QUrl ("http://"+CurrentDomain+"/") );
        
    }
}







void Gui_Main::AppendAliasCurrent() 
{
    if (CurrentDomain.size() > 3) {
         QString queryx = QString("INSERT INTO DNSEntry VALUES(NULL,'%1','A','www','','127.0.0.1','','');")
         .arg(CurrentDomain);
        
           bool oks = dbs->BaseQuery(queryx);
                     if (!oks) {
                       QMessageBox::critical(0, tr("SQLITE ERROR!"),tr("Unable to append data on  item %1 check data!").arg(CurrentDomain), QMessageBox::Cancel);
                       return;
                     }
                     
          FillNowHost(CurrentDomain);
    }
}



void Gui_Main::AllSerialUpdate()
{
   uint nummerserial = dbs->direct_int(serial->text());
    
    if (nummerserial > 300 && CurrentDomain.size() > 3 ) {
        QDateTime fromunix = QDateTime::currentDateTime();
        uint timeunixnow = fromunix.toTime_t();
        
        ///////////fromunix.toString("yyyyMMddhh")
        
        dbs->UpdateUnion("serial",timeunixnow);
        FillNowHost(CurrentDomain);
    }
    
    
}

void Gui_Main::UpdateCurrentDomain()
{
    if (CurrentDomain.size() > 3 && ACTUALIDnummer !=0 ) {
        
        ////////////std::cout << "save action  " << qPrintable(CurrentDomain) << std::endl;

        QStringList Xupdate;
                    Xupdate.clear();
        
                    QString xcomemnt = comment->toPlainText();
                             xcomemnt  =  xcomemnt.replace("'","`");
                             xcomemnt  =  xcomemnt.replace("\"","");
        
                    Xupdate.append(QString("ttl_time='%1'").arg(ttl_uint->text()));
                    Xupdate.append(QString("expirytime='%1'").arg(exp_uint->text()));
                    Xupdate.append(QString("refreshtime='%1'").arg(ref_uint->text()));
                    Xupdate.append(QString("transfer='%1'").arg(tran_uint->text()));
                    Xupdate.append(QString("master='%1'").arg(M_name->text()));
                    Xupdate.append(QString("mail='%1'").arg(dns_mail->text()));
                    Xupdate.append(QString("note='%1'").arg(xcomemnt));
               
                     QString lineux = Xupdate.join(",");
                     lineux.prepend("UPDATE SoaRegister SET ");
                     lineux.append(QString(" WHERE id=%1 ").arg(ACTUALIDnummer));
                     bool oks = dbs->BaseQuery(lineux);
                     if (!oks) {
                       QMessageBox::critical(0, tr("SQLITE ERROR!"),tr("Unable to update item %1 check data!").arg(CurrentDomain), QMessageBox::Cancel);
                       return;
                     }
        

                if (checkBox->isChecked()) {
                    dbs->UpdateUnion("ttl_time",(uint)dbs->direct_int(ttl_uint->text()));
                    ///////////std::cout << "ttl union  " << qPrintable(CurrentDomain) << std::endl; 
                }
                if (checkBox_2->isChecked()) {
                    dbs->UpdateUnion("expirytime",(uint)dbs->direct_int(exp_uint->text()));
                    dbs->UpdateUnion("refreshtime",(uint)dbs->direct_int(ref_uint->text()));
                    dbs->UpdateUnion("transfer",(uint)dbs->direct_int(tran_uint->text()));
                    /////////std::cout << "3 union  " << qPrintable(CurrentDomain) << std::endl; 
                }
                
                AllSerialUpdate();
    }
}

/* reset forms to zero */
void Gui_Main::ResetFormHost()
{
   FormsChecks(false);
   D_name->setReadOnly(false);
   D_name->setText("");
    M_name->setText("");
    ttl_uint->setText("");
   ref_uint->setText("");
    exp_uint->setText("");
    tran_uint->setText("");
    dns_mail->setText("");
    comment->setText("");
    serial->setText("");
    label_4->setText("");
    ////////////label_12->setText("");
    label_13->setText("");
    bind_text->clear();
    dbs->Paint_Table("NULLER",tableView);
    CanResolve(false);
    
}



void Gui_Main::FillNowHost( const QString host )
{
   FormsChecks(true);
   D_name->setReadOnly(true);
   D_name->setText(host);
    
    CanResolve(false);
    label_13->setText(tr("Host %1 resolve to:").arg(host));
    
   label_4->setText(tr("New Item to %1").arg(host));
         const QString selecthost = QString("SELECT  ttl_time,expirytime,refreshtime,serial,mail,master,note,transfer,id FROM SoaRegister WHERE name ='%1' ").arg(host);
         /////////////std::cout << "selecthost.. " << qPrintable(host) << std::endl;   
        const QString resolverr = QString("SELECT hasip,has FROM NameResolv WHERE name ='%1' ").arg(host);
    
         QSqlQuery query(dbs->CurrentDB());
         bool ok = query.exec(selecthost);
         comment->clear();
         Q_ASSERT(ok);
                          while (query.next()) {
                                M_name->setText(query.value(5).toString());
                                ttl_uint->setText(query.value(0).toString());
                                ref_uint->setText(query.value(2).toString());
                                exp_uint->setText(query.value(1).toString());
                                tran_uint->setText(query.value(7).toString());
                                dns_mail->setText(query.value(4).toString());
                                comment->setText(query.value(6).toString());
                                serial->setText(query.value(3).toString());
                                ACTUALIDnummer = query.value(8).toInt();
                          }
                                                
        ok = query.exec(resolverr);
        Q_ASSERT(ok);
                          
                        while (query.next()) {
                                   CanResolve(query.value(1).toInt());
                                   lineEdit->setText(query.value(0).toString());
                        }                            
                          
                          
          ////////////// std::cout << "ACTUALIDnummer.. " << ACTUALIDnummer << std::endl;
                          
                          
         QString bindconfig = dbs->BindFormat(ACTUALIDnummer);
         bind_text->clear();
         bind_text->setText(bindconfig);
                          
        QStringList masterlist = dbs->UniqueList("SELECT master FROM SoaRegister order by master");
        QCompleter *completer = new QCompleter(masterlist, this);
        completer->setCaseSensitivity(Qt::CaseInsensitive);       
        M_name->setCompleter(completer);
        QStringList maillist = dbs->UniqueList("SELECT mail FROM SoaRegister order by mail");
        QCompleter *completer1 = new QCompleter(maillist, this);
        completer1->setCaseSensitivity(Qt::CaseInsensitive);
        dns_mail->setCompleter(completer1);
        QStringList ttllist = dbs->UniqueList("SELECT ttl_time FROM SoaRegister order by ttl_time");
        QCompleter *completer2 = new QCompleter(ttllist, this);
        completer2->setCaseSensitivity(Qt::CaseInsensitive);
        ttl_uint->setCompleter(completer2);
        dbs->Paint_Table(host,tableView);
                                  
}


void Gui_Main::FillMasterBindDir()
{
    
    QString dirfromfile = QFileDialog::getExistingDirectory(this, tr("Open Dir to write BIND file."),QString(setter.value("LastDir").toString()),QFileDialog::ShowDirsOnly); 
    dirfromfile = dirfromfile.append("/");
    setter.setValue("LastDir",dirfromfile);
    setter.setValue("MasterDir",dirfromfile);
    
    
    
    
    if (dirfromfile.size() > 4) {
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));

    const QString seleclist = QString("SELECT  name,id FROM SoaRegister ");
    QSqlQuery query(dbs->CurrentDB());
    bool ok = query.exec(seleclist);
     Q_ASSERT(ok);
        
        
                          while (query.next()) {
                                   int idbs = query.value(1).toInt();
                                    QString doma = query.value(0).toString();
                                    QString filego = dirfromfile + doma + ".hosts";
                                    QString bindconfig = dbs->BindFormat(idbs);
                                    fwriteiso(filego,bindconfig);
                          }
                          
      
                          
    QApplication::restoreOverrideCursor();
    
    }
}


void Gui_Main::FillMasterListFile()
{
     bool notifics = false;
     bool ok;
     QString msgDB =tr("Insert the Notify Slave Yes or No?");
      int removeyes = QMessageBox::question(this, tr("Pleas confirm"),msgDB,
                                                            tr("&Yes"), tr("&No"),
                                                             QString(),8888,9999);
    
      if (removeyes == 0) {
        notifics = true;
      }
    
    
    const QString seleclist = QString("SELECT  name FROM SoaRegister");
    QSqlQuery query(dbs->CurrentDB());
      ok = query.exec(seleclist);
     Q_ASSERT(ok);
      
      QString path = QString(setter.value("MasterDir").toString());
      
        path = QInputDialog::getText(this, tr("Use File path"),tr("Full Path:"), QLineEdit::Normal,path, &ok);
      
      if (!path.endsWith("/")) {
       path = path.append("/");
      }
    
      
      QDateTime fromunix = QDateTime::currentDateTime();
      QString update = fromunix.toString("yyyyMMddhh");
      
      QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
      
      QString bin = "# Master dns server list to include. \n";
              bin += "# \n";
              bin += QString("# UPDATE AT %1 BY %2 \n\n").arg(update).arg(_PROGRAM_TITLE_ );
      
      while (query.next()) {
                  QString doma = query.value(0).toString();
                  QString filego = path + doma + ".hosts";
           bin += QString("zone \"%1\" {\n").arg(doma);
           bin += QString("type master; \n");
           bin += QString("file \"%2%1.hosts\";\n").arg(doma).arg(path);
           if (notifics) {
           bin += QString("notify yes; \n");
           }
           bin += QString("}; \n\n\n");
          
      }
      
      QApplication::restoreOverrideCursor();
      
    
      QString fullFileName = QFileDialog::getSaveFileName(this, tr("Write result to file..."),QString(setter.value("LastMaster").toString()), "*");
      if (fullFileName.size() > 1) {
      setter.setValue("LastMaster",fullFileName);
      QString ultimacartellaaperta = fullFileName.left(fullFileName.lastIndexOf("/"))+"/";
      setter.setValue("LastDir",ultimacartellaaperta);
      fwriteiso(fullFileName,bin);
      }
      
      
}


void Gui_Main::FillSlaveListFile()
{
    QMessageBox::information(0, qApp->tr("Try to resolv ips from master.."),tr("Please wait to resolve master ip server, close now this."), QMessageBox::Cancel);
    ResolveAllName();
    QTimer::singleShot(2000, this, SLOT(FillSlaveGo()));
}

void Gui_Main::FillSlaveGo()
{
       bool ok;
       QString path = QString(setter.value("MasterDir").toString());
        path = QInputDialog::getText(this, tr("Use File path"),tr("Slave Full Path:"), QLineEdit::Normal,path, &ok);
    
      if (!path.endsWith("/")) {
       path = path.append("/");
      }
      
      QDateTime fromunix = QDateTime::currentDateTime();
      QString update = fromunix.toString("yyyyMMddhh");
      
      QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
      
      QString bin = "# Slave dns server list to include. \n";
              bin += "# \n";
              bin += QString("# UPDATE AT %1 BY %2 \n\n").arg(update).arg(_PROGRAM_TITLE_ );
      
      
      
    const QString seleclist = QString("SELECT  name,master FROM SoaRegister");
    QSqlQuery query(dbs->CurrentDB());
    ok = query.exec(seleclist);
     Q_ASSERT(ok);
          while (query.next()) {
                  QString doma = query.value(0).toString();
                  QString masterip = dbs->IPsCacheFrom(query.value(1).toString());
                  QString filego = path + doma + ".hosts";
              
           bin += QString("zone \"%1\" {\n").arg(doma);
           bin += QString("type slave; \n");
           bin += QString("masters {\n\t%1;\n\t};\n").arg(masterip);
           bin += QString("file \"%2%1.hosts\";\n").arg(doma).arg(path);
           bin += QString("}; \n\n\n");
         }
         
         
         
    QApplication::restoreOverrideCursor();
         
      QString fullFileName = QFileDialog::getSaveFileName(this, tr("Write result to file..."),QString(setter.value("LastSlave").toString()), "*");
      if (fullFileName.size() > 1) {
      setter.setValue("LastSlave",fullFileName);
      QString ultimacartellaaperta = fullFileName.left(fullFileName.lastIndexOf("/"))+"/";
      setter.setValue("LastDir",ultimacartellaaperta);
      fwriteiso(fullFileName,bin);
      } 
         
         
}

void Gui_Main::SqliteExport() 
{
    
      QString fullFileName = QFileDialog::getSaveFileName(this, tr("Write result to file..."),QString(setter.value("LastDir").toString()), "*.sql");
      if (fullFileName.size() > 1) {
          
          if (!fullFileName.endsWith(".sql")) {
              fullFileName.append(".sql");
          }
          
      QString ultimacartellaaperta = fullFileName.left(fullFileName.lastIndexOf("/"))+"/";
      setter.setValue("LastDir",ultimacartellaaperta);
          QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
          const QString bin = dbs->ExportTbs();
          
          fwriteiso(fullFileName,bin);
          QApplication::restoreOverrideCursor();
          
      } 
}

void Gui_Main::SqliteImport()
{
     QString fullFileName = QFileDialog::getOpenFileName(this, tr("Open sql file file..."),QString(setter.value("LastDir").toString()), "*.sql");
      if (fullFileName.size() > 1) {
      QString ultimacartellaaperta = fullFileName.left(fullFileName.lastIndexOf("/"))+"/";
      setter.setValue("LastDir",ultimacartellaaperta);
      QFile file(fullFileName); 
          
               if( !file.open( QIODevice::ReadOnly | QIODevice::Text ) ) {
                   return;
                }
                QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
                
                QTextStream in( &file );
                 while( ! in.atEnd() ) {
                     QString line( in.readLine() );
                              line = line.trimmed();
                     if (line.startsWith("#")) {
                      continue;
                     }
                     if (line.contains("REPLACE")) {
                        dbs->BaseQuery(line);
                     }
                 }
                 LoadDomainHosts();
                 QApplication::restoreOverrideCursor();
          
      }
    
    
    
}



void Gui_Main::NewDomaine()
{
    
     bool ok;
       QString nehost = "domaine.com";
        nehost = QInputDialog::getText(this, tr("New Dns Domain"),tr("Name:"), QLineEdit::Normal,nehost, &ok);
    
       if (nehost.size() > 1) {
           
           const QString hdom = GrepHostName(nehost);
           if (hdom.size() > 1) {
               
               QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
               dbs->BaseQuery(QString("INSERT INTO SoaRegister VALUES (NULL,'%1','300','nic@%1','ns0.%1','1183398837','300','300','1183403071','3600','%1.hosts','Take from user')").arg(hdom));
               dbs->BaseQuery(QString("INSERT INTO DNSEntry VALUES (NULL,'%1','NS',NULL,'300','localhost1',NULL,NULL)").arg(hdom));
               dbs->BaseQuery(QString("INSERT INTO DNSEntry VALUES (NULL,'%1','NS',NULL,'300','localhost2',NULL,NULL)").arg(hdom));
               dbs->BaseQuery(QString("INSERT INTO DNSEntry VALUES (NULL,'%1','A',NULL,'300','127.0.0.1',NULL,NULL)").arg(hdom));
               dbs->BaseQuery(QString("INSERT INTO DNSEntry VALUES (NULL,'%1','MX',NULL,'300','mail.localhost.com','5',NULL)").arg(hdom));
               dbs->BaseQuery(QString("INSERT INTO DNSEntry VALUES (NULL,'%1','CNAME','www','300','%1',NULL,NULL)").arg(hdom));
               LoadDomainHosts();
               QApplication::restoreOverrideCursor();
               
           }
           
       }
    
    
}





void Gui_Main::FormsChecks( bool activate )
{
    D_name->setEnabled(activate);
    M_name->setEnabled(activate);
    ttl_uint->setEnabled(activate);
    ref_uint->setEnabled(activate);
    exp_uint->setEnabled(activate);
    tran_uint->setEnabled(activate);
    dns_mail->setEnabled(activate);
    comment->setEnabled(activate);
    serial->setEnabled(activate);
    
}



void Gui_Main::closeEvent( QCloseEvent* e )
{
	e->accept();
}


void Gui_Main::AboutApp()
{
    Gui_About *mani = new Gui_About(this);
    mani->show();
    
}



void Gui_Main::ErrorImportBind( const QString msg )
{
    QMessageBox::critical(0, qApp->tr("Cannot import domain from bind "),msg, QMessageBox::Cancel);
    
}



void Gui_Main::CanResolve( bool e )
{
    
    QPixmap pix(22,22);
    if (e) {
    pix.fill(QColor("lime"));
    } else {
    pix.fill(QColor("crimson"));  
    }
    label_12->setPixmap(pix);
}


void Gui_Main::OpenDesktop( QUrl loc )
{ 
    //////////qDebug() << "#### OpenDesktop " << loc; 
    
#if defined Q_WS_WIN
    
QString winbuh = DesktopFile(loc.toString());
      /* QUrl(winbuh) */
    
    bool wr = QDesktopServices::openUrl(loc);
    if (!wr) {
    QMessageBox::warning(0, tr("Error"),tr("Unable to open file or dir  %1").arg(loc.toString()));
    }
return;
#endif
    
    
     bool r = QDesktopServices::openUrl(loc);
    if (!r) {
    QMessageBox::warning(0, tr("Error"),tr("Unable to open file or dir  %1").arg(loc.toString()));
    }
}

/* Q_ASSERT ( bool test )  */ 

/* QString one;  toLower() toUpper()  split(",")  endsWith() startsWith()  QByteArray QString::toAscii() */
/* int nummer; */ 
/* QStringList list;  join();  */ 
/* QUrl url; */ 
/* QByteArray byteArray; */ 
/* bool having; */ 
/* QDomDocument doc;*/ 
/* QDomElement root;*/ 
